@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -68,6 +68,7 @@ DWORD cm_mountRootLen;
 clientchar_t cm_mountRootC[1024];
 DWORD cm_mountRootCLen;
 
+int cm_readonlyVolumeVersioning = 0;
 int cm_logChunkSize;
 int cm_chunkSize;
 
@@ -112,9 +113,7 @@ DWORD TraceOption = 0;
 
 HANDLE afsi_file;
 
-#ifdef AFS_AFSDB_ENV
 int cm_dnsEnabled = 1;
-#endif
 
 
 static int afsi_log_useTimestamp = 1;
@@ -136,7 +135,7 @@ afsi_log(char *pattern, ...)
             WriteFile(afsi_file, u, (DWORD)strlen(u), &zilch, NULL);
 #ifdef NOTSERVICE
         printf("%s", u);
-#endif 
+#endif
     } else {
         if (afsi_file != INVALID_HANDLE_VALUE)
             WriteFile(afsi_file, s, (DWORD)strlen(s), &zilch, NULL);
@@ -203,10 +202,10 @@ afsi_start()
     afsi_log("OEM Code Page = %d", GetOEMCP());
     afsi_log("locale =  %s", setlocale(LC_ALL,NULL));
 #ifdef COMMENT
-    /* Two things to look into.  First, should mbstowcs() be performing 
-     * character set translations from OEM to Unicode in smb3.c; 
-     * Second, do we need to set this translation in each function 
-     * due to multi-threading. 
+    /* Two things to look into.  First, should mbstowcs() be performing
+     * character set translations from OEM to Unicode in smb3.c;
+     * Second, do we need to set this translation in each function
+     * due to multi-threading.
      */
     afsi_log("locale -> %s", setlocale(LC_ALL, ".OCP"));
     afsi_log("_setmbcp = %d -> %d", _setmbcp(_MB_CP_OEM), _getmbcp());
@@ -223,7 +222,7 @@ void afsd_ForceTrace(BOOL flush)
     int len;
     char buf[256];
 
-    if (!logReady) 
+    if (!logReady)
         return;
 
     len = GetTempPath(sizeof(buf)-10, buf);
@@ -252,7 +251,7 @@ static void afsd_InitServerPreferences(void)
     struct sockaddr_in	saddr;
     cm_server_t       *tsp;
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Server Preferences\\VLDB",
                       0,
                       KEY_READ|KEY_QUERY_VALUE,
@@ -297,15 +296,17 @@ static void afsd_InitServerPreferences(void)
 
                 saddr.sin_addr.S_un.S_addr = *(unsigned long *)pEntry->h_addr;
             }
+            saddr.sin_port = htons(7003);
             saddr.sin_family = AF_INET;
             dwRank += (rand() & 0x000f);
 
-            tsp = cm_FindServer(&saddr, CM_SERVER_VLDB);
+            tsp = cm_FindServer(&saddr, CM_SERVER_VLDB, FALSE);
             if ( tsp )		/* an existing server - ref count increased */
             {
                 lock_ObtainMutex(&tsp->mx);
                 tsp->ipRank = (USHORT)dwRank;
-                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+                _InterlockedOr(&tsp->flags, CM_SERVERFLAG_PREF_SET);
+		tsp->adminRank = tsp->ipRank;
                 lock_ReleaseMutex(&tsp->mx);
 
                 /* set preferences for an existing vlserver */
@@ -317,7 +318,8 @@ static void afsd_InitServerPreferences(void)
                 tsp = cm_NewServer(&saddr, CM_SERVER_VLDB, NULL, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
                 lock_ObtainMutex(&tsp->mx);
                 tsp->ipRank = (USHORT)dwRank;
-                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+                _InterlockedOr(&tsp->flags, CM_SERVERFLAG_PREF_SET);
+		tsp->adminRank = tsp->ipRank;
                 lock_ReleaseMutex(&tsp->mx);
             }
         }
@@ -325,7 +327,7 @@ static void afsd_InitServerPreferences(void)
         RegCloseKey(hkPrefs);
     }
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Server Preferences\\File",
                       0,
                       KEY_READ|KEY_QUERY_VALUE,
@@ -370,20 +372,22 @@ static void afsd_InitServerPreferences(void)
 
                 saddr.sin_addr.S_un.S_addr = *(unsigned long *)pEntry->h_addr;
             }
+            saddr.sin_port = htons(7000);
             saddr.sin_family = AF_INET;
             dwRank += (rand() & 0x000f);
 
-            tsp = cm_FindServer(&saddr, CM_SERVER_FILE);
+            tsp = cm_FindServer(&saddr, CM_SERVER_FILE, FALSE);
             if ( tsp )		/* an existing server - ref count increased */
             {
                 lock_ObtainMutex(&tsp->mx);
                 tsp->ipRank = (USHORT)dwRank;
-                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+		_InterlockedOr(&tsp->flags, CM_SERVERFLAG_PREF_SET);
+		tsp->adminRank = tsp->ipRank;
                 lock_ReleaseMutex(&tsp->mx);
 
-                /* find volumes which might have RO copy 
-                /* on server and change the ordering of 
-                 * their RO list 
+                /* find volumes which might have RO copy
+                /* on server and change the ordering of
+                 * their RO list
                  */
                 cm_ChangeRankVolume(tsp);
                 cm_PutServer(tsp);  /* decrease refcount */
@@ -393,7 +397,8 @@ static void afsd_InitServerPreferences(void)
                 tsp = cm_NewServer(&saddr, CM_SERVER_FILE, NULL, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
                 lock_ObtainMutex(&tsp->mx);
                 tsp->ipRank = (USHORT)dwRank;
-                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+                _InterlockedOr(&tsp->flags, CM_SERVERFLAG_PREF_SET);
+		tsp->adminRank = tsp->ipRank;
                 lock_ReleaseMutex(&tsp->mx);
             }
         }
@@ -498,6 +503,9 @@ afsd_InitCM(char **reasonP)
     long maxcpus;
     long ltt, ltto;
     long rx_nojumbo;
+    int  rx_max_rwin_size;
+    int  rx_max_swin_size;
+    int  rx_min_peer_timeout;
     long virtualCache = 0;
     fschar_t rootCellName[256];
     struct rx_service *serverp;
@@ -657,6 +665,11 @@ afsd_InitCM(char **reasonP)
 
     osi_Log0(afsd_logp, "Log init");
 
+    dummyLen = sizeof(smb_monitorReqs);
+    code = RegQueryValueEx(parmKey, "SMBRequestMonitor", NULL, NULL,
+                           (BYTE *) &smb_monitorReqs, &dummyLen);
+    afsi_log("SMB request monitoring is %s", (smb_monitorReqs != 0)? "enabled": "disabled");
+
     dummyLen = sizeof(cacheSize);
     code = RegQueryValueEx(parmKey, "CacheSize", NULL, NULL,
                             (BYTE *) &cacheSize, &dummyLen);
@@ -687,13 +700,13 @@ afsd_InitCM(char **reasonP)
     code = RegQueryValueEx(parmKey, "blockSize", NULL, NULL,
                             (BYTE *) &blockSize, &dummyLen);
     if (code == ERROR_SUCCESS) {
-        if (blockSize < 1 || 
-            (blockSize > 1024 && (blockSize % CM_CONFIGDEFAULT_BLOCKSIZE != 0))) 
+        if (blockSize < 1 ||
+            (blockSize > 1024 && (blockSize % CM_CONFIGDEFAULT_BLOCKSIZE != 0)))
         {
             afsi_log("Invalid block size %u specified, using default", blockSize);
             blockSize = CM_CONFIGDEFAULT_BLOCKSIZE;
         } else {
-            /* 
+            /*
              * if the blockSize is less than 1024 we permit the blockSize to be
              * specified in multiples of the default blocksize
              */
@@ -704,7 +717,7 @@ afsd_InitCM(char **reasonP)
         blockSize = CM_CONFIGDEFAULT_BLOCKSIZE;
     }
     if (blockSize > cm_chunkSize) {
-        afsi_log("Block size (%d) cannot be larger than Chunk size (%d).", 
+        afsi_log("Block size (%d) cannot be larger than Chunk size (%d).",
                   blockSize, cm_chunkSize);
         blockSize = cm_chunkSize;
     }
@@ -783,7 +796,7 @@ afsd_InitCM(char **reasonP)
             ltto = 120;
     } else {
         ltto = 0;
-    }   
+    }
     smb_LogoffTransferTimeout = ltto;
     afsi_log("Logoff token transfer timeout %d seconds", ltto);
 
@@ -829,7 +842,7 @@ afsd_InitCM(char **reasonP)
     } else {
         dummyLen = ExpandEnvironmentStrings("%TEMP%\\AFSCache", cm_CachePath, sizeof(cm_CachePath));
         if (dummyLen > sizeof(cm_CachePath)) {
-            afsi_log("Cache path [%%TEMP%%\\AFSCache] longer than %d after expanding env strings", 
+            afsi_log("Cache path [%%TEMP%%\\AFSCache] longer than %d after expanding env strings",
                      sizeof(cm_CachePath));
             osi_panic("CachePath too long", __FILE__, __LINE__);
         }
@@ -898,7 +911,7 @@ afsd_InitCM(char **reasonP)
             cm_ClientStrCpy(cbuf, lengthof(buf), _C("x86_win32 i386_w2k i386_nt40"));
 #endif
         }
-        afsi_log("Sys name %S", cbuf); 
+        afsi_log("Sys name %S", cbuf);
 
         /* breakup buf into individual search string entries */
         for (p = q = cbuf; p < cbuf + dummyLen; p++) {
@@ -941,33 +954,30 @@ afsd_InitCM(char **reasonP)
                             (BYTE *) &cm_anonvldb, &dummyLen);
     afsi_log("CM ForceAnonVLDB is %s", cm_anonvldb ? "on" : "off");
 
-#ifdef AFS_AFSDB_ENV
     dummyLen = sizeof(cm_dnsEnabled);
     code = RegQueryValueEx(parmKey, "UseDNS", NULL, NULL,
                             (BYTE *) &cm_dnsEnabled, &dummyLen);
     if (code == ERROR_SUCCESS) {
         afsi_log("DNS %s be used to find AFS cell servers",
                   cm_dnsEnabled ? "will" : "will not");
-    }       
+    }
     else {
         cm_dnsEnabled = 1;   /* default on */
         afsi_log("Default to use DNS to find AFS cell servers");
     }
-#else /* AFS_AFSDB_ENV */
-    afsi_log("AFS not built with DNS support to find AFS cell servers");
-#endif /* AFS_AFSDB_ENV */
 
 #ifdef AFS_FREELANCE_CLIENT
     dummyLen = sizeof(cm_freelanceEnabled);
     code = RegQueryValueEx(parmKey, "FreelanceClient", NULL, NULL,
                             (BYTE *) &cm_freelanceEnabled, &dummyLen);
-    if (code == ERROR_SUCCESS) {
-        afsi_log("Freelance client feature %s activated",
-                  cm_freelanceEnabled ? "is" : "is not");
-    }       
-    else {
-        cm_freelanceEnabled = 1;  /* default on */
-    }
+    afsi_log("Freelance client feature %s activated",
+              cm_freelanceEnabled ? "is" : "is not");
+
+    dummyLen = sizeof(cm_freelanceImportCellServDB);
+    code = RegQueryValueEx(parmKey, "FreelanceImportCellServDB", NULL, NULL,
+                            (BYTE *) &cm_freelanceImportCellServDB, &dummyLen);
+    afsi_log("Freelance client %s import CellServDB",
+              cm_freelanceImportCellServDB ? "does" : "does not");
 #endif /* AFS_FREELANCE_CLIENT */
 
     dummyLen = sizeof(smb_UseUnicode);
@@ -988,6 +998,22 @@ afsd_InitCM(char **reasonP)
     afsi_log("Dot files/dirs will %sbe marked hidden",
               smb_hideDotFiles ? "" : "not ");
 
+    dummyLen = sizeof(dwValue);
+    code = RegQueryValueEx(parmKey, "UnixModeFileDefault", NULL, NULL,
+                           (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS) {
+        smb_unixModeDefaultFile = (dwValue & 07777);
+    }
+    afsi_log("Default unix mode bits for files is 0%04o", smb_unixModeDefaultFile);
+
+    dummyLen = sizeof(dwValue);
+    code = RegQueryValueEx(parmKey, "UnixModeDirDefault", NULL, NULL,
+                           (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS) {
+        smb_unixModeDefaultDir = (dwValue & 07777);
+    }
+    afsi_log("Default unix mode bits for directories is 0%04o", smb_unixModeDefaultDir);
+
     dummyLen = sizeof(smb_maxMpxRequests);
     code = RegQueryValueEx(parmKey, "MaxMpxRequests", NULL, NULL,
                            (BYTE *) &smb_maxMpxRequests, &dummyLen);
@@ -1008,12 +1034,38 @@ afsd_InitCM(char **reasonP)
     code = RegQueryValueEx(parmKey, "SMBAuthType", NULL, NULL,
                             (BYTE *) &smb_authType, &dummyLen);
 
-    if (code != ERROR_SUCCESS || 
+    if (code != ERROR_SUCCESS ||
          (smb_authType != SMB_AUTH_EXTENDED && smb_authType != SMB_AUTH_NTLM && smb_authType != SMB_AUTH_NONE)) {
         smb_authType = SMB_AUTH_EXTENDED; /* default is to use extended authentication */
     }
     afsi_log("SMB authentication type is %s", ((smb_authType == SMB_AUTH_NONE)?"NONE":((smb_authType == SMB_AUTH_EXTENDED)?"EXTENDED":"NTLM")));
 
+    dummyLen = sizeof(rx_max_rwin_size);
+    code = RegQueryValueEx(parmKey, "RxMaxRecvWinSize", NULL, NULL,
+                           (BYTE *) &rx_max_rwin_size, &dummyLen);
+    if (code == ERROR_SUCCESS)
+        rx_SetMaxReceiveWindow(rx_max_rwin_size);
+    afsi_log("Rx Maximum Receive Window Size is %d", rx_GetMaxReceiveWindow());
+
+    dummyLen = sizeof(rx_max_swin_size);
+    code = RegQueryValueEx(parmKey, "RxMaxSendWinSize", NULL, NULL,
+                           (BYTE *) &rx_max_swin_size, &dummyLen);
+    if (code == ERROR_SUCCESS)
+        rx_SetMaxSendWindow(rx_max_swin_size);
+    afsi_log("Rx Maximum Send Window Size is %d", rx_GetMaxSendWindow());
+
+    dummyLen = sizeof(rx_min_peer_timeout);
+    code = RegQueryValueEx(parmKey, "RxMinPeerTimeout", NULL, NULL,
+                           (BYTE *) &rx_min_peer_timeout, &dummyLen);
+    if (code == ERROR_SUCCESS)
+        rx_SetMinPeerTimeout(rx_min_peer_timeout);
+    afsi_log("Rx Minimum Peer Timeout is %d ms", rx_GetMinPeerTimeout());
+
+    dummyLen = sizeof(rx_pmtu_discovery);
+    code = RegQueryValueEx(parmKey, "RxPMTUDiscovery", NULL, NULL,
+                           (BYTE *) &rx_pmtu_discovery, &dummyLen);
+    afsi_log("Rx PMTU Discovery is %d ms", rx_pmtu_discovery);
+
     dummyLen = sizeof(rx_nojumbo);
     code = RegQueryValueEx(parmKey, "RxNoJumbo", NULL, NULL,
                            (BYTE *) &rx_nojumbo, &dummyLen);
@@ -1106,7 +1158,7 @@ afsd_InitCM(char **reasonP)
                            (BYTE *) &dwValue, &dummyLen);
     if (code == ERROR_SUCCESS) {
         cm_enableServerLocks = (unsigned short) dwValue;
-    } 
+    }
     switch (cm_enableServerLocks) {
     case 0:
 	afsi_log("EnableServerLocks: never");
@@ -1125,16 +1177,16 @@ afsd_InitCM(char **reasonP)
                            (BYTE *) &dwValue, &dummyLen);
     if (code == ERROR_SUCCESS) {
         cm_deleteReadOnly = (unsigned short) dwValue;
-    } 
+    }
     afsi_log("CM DeleteReadOnly is %u", cm_deleteReadOnly);
-    
+
 #ifdef USE_BPLUS
     dummyLen = sizeof(DWORD);
     code = RegQueryValueEx(parmKey, "BPlusTrees", NULL, NULL,
                            (BYTE *) &dwValue, &dummyLen);
     if (code == ERROR_SUCCESS) {
         cm_BPlusTrees = (unsigned short) dwValue;
-    } 
+    }
     afsi_log("CM BPlusTrees is %u", cm_BPlusTrees);
 
     if (cm_BPlusTrees && !cm_InitBPlusDir()) {
@@ -1145,14 +1197,14 @@ afsd_InitCM(char **reasonP)
     afsi_log("CM BPlusTrees is not supported");
 #endif
 
-    if ((RegQueryValueExW( parmKey, L"PrefetchExecutableExtensions", 0, 
+    if ((RegQueryValueExW( parmKey, L"PrefetchExecutableExtensions", 0,
                            &regType, NULL, &dummyLen) == ERROR_SUCCESS) &&
-         (regType == REG_MULTI_SZ)) 
+         (regType == REG_MULTI_SZ))
     {
         clientchar_t * pSz;
         dummyLen += 3; /* in case the source string is not nul terminated */
         pSz = malloc(dummyLen);
-        if ((RegQueryValueExW( parmKey, L"PrefetchExecutableExtensions", 0, &regType, 
+        if ((RegQueryValueExW( parmKey, L"PrefetchExecutableExtensions", 0, &regType,
                                (LPBYTE) pSz, &dummyLen) == ERROR_SUCCESS) &&
              (regType == REG_MULTI_SZ))
         {
@@ -1169,7 +1221,7 @@ afsd_InitCM(char **reasonP)
             }
             smb_ExecutableExtensions[cnt] = NULL;
         }
-        
+
         if (!smb_ExecutableExtensions)
             free(pSz);
     }
@@ -1181,15 +1233,15 @@ afsd_InitCM(char **reasonP)
                            (BYTE *) &dwValue, &dummyLen);
     if (code == ERROR_SUCCESS) {
         cm_OfflineROIsValid = (unsigned short) dwValue;
-    } 
-    afsi_log("CM OfflineReadOnlyIsValid is %u", cm_deleteReadOnly);
-    
+    }
+    afsi_log("CM OfflineReadOnlyIsValid is %u", cm_OfflineROIsValid);
+
     dummyLen = sizeof(DWORD);
     code = RegQueryValueEx(parmKey, "GiveUpAllCallBacks", NULL, NULL,
                            (BYTE *) &dwValue, &dummyLen);
     if (code == ERROR_SUCCESS) {
         cm_giveUpAllCBs = (unsigned short) dwValue;
-    } 
+    }
     afsi_log("CM GiveUpAllCallBacks is %u", cm_giveUpAllCBs);
 
     dummyLen = sizeof(DWORD);
@@ -1197,7 +1249,7 @@ afsd_InitCM(char **reasonP)
                            (BYTE *) &dwValue, &dummyLen);
     if (code == ERROR_SUCCESS) {
         cm_followBackupPath = (unsigned short) dwValue;
-    } 
+    }
     afsi_log("CM FollowBackupPath is %u", cm_followBackupPath);
 
     dummyLen = sizeof(DWORD);
@@ -1205,13 +1257,21 @@ afsd_InitCM(char **reasonP)
                            (BYTE *) &dwValue, &dummyLen);
     if (code == ERROR_SUCCESS) {
         cm_accessPerFileCheck = (int) dwValue;
-    } 
+    }
     afsi_log("CM PerFileAccessCheck is %d", cm_accessPerFileCheck);
 
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "ReadOnlyVolumeVersioning", NULL, NULL,
+                           (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS) {
+        cm_readonlyVolumeVersioning = (unsigned short) dwValue;
+    }
+    afsi_log("CM ReadOnlyVolumeVersioning is %u", cm_readonlyVolumeVersioning);
+
     RegCloseKey (parmKey);
 
     cacheBlocks = ((afs_uint64)cacheSize * 1024) / blockSize;
-        
+
     /* get network related info */
     cm_noIPAddr = CM_MAXINTERFACE_ADDR;
     code = syscfg_GetIFInfo(&cm_noIPAddr,
@@ -1243,11 +1303,11 @@ afsd_InitCM(char **reasonP)
     cm_InitConn();
 
     cm_InitServer();
-        
+
     cm_InitIoctl();
-        
+
     smb_InitIoctl();
-        
+
     cm_InitCallback();
 
     cm_InitNormalization();
@@ -1259,12 +1319,10 @@ afsd_InitCM(char **reasonP)
         return -1;
     }
 
-#ifdef AFS_AFSDB_ENV
 #if !defined(_WIN32_WINNT) || (_WIN32_WINNT < 0x0500)
     if (cm_InitDNS(cm_dnsEnabled) == -1)
         cm_dnsEnabled = 0;  /* init failed, so deactivate */
     afsi_log("cm_InitDNS %d", cm_dnsEnabled);
-#endif
 #endif
 
     /* Set RX parameters before initializing RX */
@@ -1285,7 +1343,9 @@ afsd_InitCM(char **reasonP)
         afsi_log("rx_SetUdpBufSize %d", rx_udpbufsize);
     }
 
-    /* initialize RX, and tell it to listen to the callbackport, 
+    rx_SetBusyChannelError(CM_RX_RETRY_BUSY_CALL);
+
+    /* initialize RX, and tell it to listen to the callbackport,
      * which is used for callback RPC messages.
      */
     code = rx_Init(htons(cm_callbackport));
@@ -1322,7 +1382,7 @@ afsd_InitCM(char **reasonP)
     }
     rx_SetMinProcs(serverp, 2);
     rx_SetMaxProcs(serverp, 4);
-        
+
     /* start server threads, *not* donating this one to the pool */
     rx_StartServer(0);
     afsi_log("rx_StartServer");
@@ -1334,21 +1394,21 @@ afsd_InitCM(char **reasonP)
 	rx_enableProcessRPCStats();
 
     code = cm_GetRootCellName(rootCellName);
-    afsi_log("cm_GetRootCellName code %d, cm_freelanceEnabled= %d, rcn= %s", 
+    afsi_log("cm_GetRootCellName code %d, cm_freelanceEnabled= %d, rcn= %s",
              code, cm_freelanceEnabled, (code ? "<none>" : rootCellName));
-    if (code != 0 && !cm_freelanceEnabled) 
+    if (code != 0 && !cm_freelanceEnabled)
     {
         *reasonP = "can't find root cell name in " AFS_CELLSERVDB;
         return -1;
-    }   
+    }
     else if (cm_freelanceEnabled)
         cm_data.rootCellp = NULL;
 
-    if (code == 0 && !cm_freelanceEnabled) 
+    if (code == 0 && !cm_freelanceEnabled)
     {
         cm_data.rootCellp = cm_GetCell(rootCellName, CM_FLAG_CREATE);
         afsi_log("cm_GetCell addr %x", PtrToUlong(cm_data.rootCellp));
-        if (cm_data.rootCellp == NULL) 
+        if (cm_data.rootCellp == NULL)
         {
             *reasonP = "can't find root cell in " AFS_CELLSERVDB;
             return -1;
@@ -1429,10 +1489,10 @@ int afsd_InitSMB(char **reasonP, void *aMBfunc)
                 smb_AsyncStoreSize = cm_data.buf_blockSize;
             else
                 smb_AsyncStoreSize = (dwValue & ~(cm_data.buf_blockSize-1));
-        } else 
+        } else
             smb_AsyncStoreSize = CM_CONFIGDEFAULT_ASYNCSTORESIZE;
         afsi_log("SMBAsyncStoreSize = %d", smb_AsyncStoreSize);
-        
+
         RegCloseKey (parmKey);
     }
 
@@ -1470,33 +1530,33 @@ void afsd_printStack(HANDLE hThread, CONTEXT *c)
 #endif
     DWORD symOptions;
     char functionName[MAXNAMELEN];
-  
+
     IMAGEHLP_MODULE Module;
     IMAGEHLP_LINE Line;
-  
+
     STACKFRAME s;
     IMAGEHLP_SYMBOL *pSym;
-  
+
     afsi_log_useTimestamp = 0;
-  
+
     pSym = (IMAGEHLP_SYMBOL *) GlobalAlloc(0, sizeof (IMAGEHLP_SYMBOL) + MAXNAMELEN);
-  
+
     memset( &s, '\0', sizeof s );
     if (!SymInitialize(hProcess, NULL, 1) )
     {
         afsi_log("SymInitialize(): GetLastError() = %lu\n", GetLastError() );
-      
+
         SymCleanup( hProcess );
         GlobalFree(pSym);
-      
+
         return;
     }
-  
+
     symOptions = SymGetOptions();
     symOptions |= SYMOPT_LOAD_LINES;
     symOptions &= ~SYMOPT_UNDNAME;
     SymSetOptions( symOptions );
-  
+
     /*
      * init STACKFRAME for first call
      * Notes: AddrModeFlat is just an assumption. I hate VDM debugging.
@@ -1521,55 +1581,55 @@ void afsd_printStack(HANDLE hThread, CONTEXT *c)
     memset( pSym, '\0', sizeof (IMAGEHLP_SYMBOL) + MAXNAMELEN );
     pSym->SizeOfStruct = sizeof (IMAGEHLP_SYMBOL);
     pSym->MaxNameLength = MAXNAMELEN;
-  
+
     memset( &Line, '\0', sizeof Line );
     Line.SizeOfStruct = sizeof Line;
-  
+
     memset( &Module, '\0', sizeof Module );
     Module.SizeOfStruct = sizeof Module;
-  
+
     offset = 0;
-  
+
     afsi_log("\n--# FV EIP----- RetAddr- FramePtr StackPtr Symbol" );
-  
+
     for ( frameNum = 0; ; ++ frameNum )
     {
         /*
-         * get next stack frame (StackWalk(), SymFunctionTableAccess(), 
+         * get next stack frame (StackWalk(), SymFunctionTableAccess(),
          * SymGetModuleBase()). if this returns ERROR_INVALID_ADDRESS (487) or
          * ERROR_NOACCESS (998), you can assume that either you are done, or
          * that the stack is so hosed that the next deeper frame could not be
          * found.
          */
-        if ( ! StackWalk( IMAGE_FILE_MACHINE_I386, hProcess, hThread, &s, c, 
-                          NULL, SymFunctionTableAccess, SymGetModuleBase, 
+        if ( ! StackWalk( IMAGE_FILE_MACHINE_I386, hProcess, hThread, &s, c,
+                          NULL, SymFunctionTableAccess, SymGetModuleBase,
                           NULL ) )
             break;
-      
+
         /* display its contents */
         afsi_log("\n%3d %c%c %08lx %08lx %08lx %08lx ",
                  frameNum, s.Far? 'F': '.', s.Virtual? 'V': '.',
                  s.AddrPC.Offset, s.AddrReturn.Offset,
                  s.AddrFrame.Offset, s.AddrStack.Offset );
-      
+
         if ( s.AddrPC.Offset == 0 )
         {
             afsi_log("(-nosymbols- PC == 0)" );
         }
         else
-        { 
+        {
             /* show procedure info from a valid PC */
             if (!SymGetSymFromAddr(hProcess, s.AddrPC.Offset, &offset, pSym))
             {
                 if ( GetLastError() != ERROR_INVALID_ADDRESS )
                 {
-                    afsi_log("SymGetSymFromAddr(): errno = %lu", 
+                    afsi_log("SymGetSymFromAddr(): errno = %lu",
                              GetLastError());
                 }
             }
             else
             {
-                UnDecorateSymbolName(pSym->Name, functionName, MAXNAMELEN, 
+                UnDecorateSymbolName(pSym->Name, functionName, MAXNAMELEN,
                                      UNDNAME_NAME_ONLY);
                 afsi_log("%s", functionName );
 
@@ -1583,17 +1643,17 @@ void afsd_printStack(HANDLE hThread, CONTEXT *c)
             {
                 if (GetLastError() != ERROR_INVALID_ADDRESS)
                 {
-                    afsi_log("Error: SymGetLineFromAddr(): errno = %lu", 
+                    afsi_log("Error: SymGetLineFromAddr(): errno = %lu",
                              GetLastError());
                 }
             }
             else
             {
-                afsi_log("    Line: %s(%lu) %+ld bytes", Line.FileName, 
+                afsi_log("    Line: %s(%lu) %+ld bytes", Line.FileName,
                          Line.LineNumber, offset);
             }
         }
-      
+
         /* no return address means no deeper stackframe */
         if (s.AddrReturn.Offset == 0)
         {
@@ -1601,12 +1661,12 @@ void afsd_printStack(HANDLE hThread, CONTEXT *c)
             break;
         }
     }
-  
+
     if (GetLastError() != 0)
     {
         afsi_log("\nStackWalk(): errno = %lu\n", GetLastError());
     }
-  
+
     SymCleanup(hProcess);
     GlobalFree(pSym);
 }
@@ -1627,37 +1687,42 @@ static BOOL (WINAPI *pMiniDumpWriteDump)(HANDLE hProcess,DWORD ProcessId,HANDLE
 static HANDLE
 OpenDumpFile(void)
 {
+    char tmp[256];
     char wd[256];
+    SYSTEMTIME st;
     DWORD code;
 
-    code = GetEnvironmentVariable("TEMP", wd, sizeof(wd));
-    if ( code == 0 || code > sizeof(wd) )
+    code = GetEnvironmentVariable("TEMP", tmp, sizeof(tmp));
+    if ( code == 0 || code > sizeof(tmp) )
     {
-        if (!GetWindowsDirectory(wd, sizeof(wd)))
+        if (!GetWindowsDirectory(tmp, sizeof(tmp)))
             return NULL;
     }
-    StringCbCatA(wd, sizeof(wd), "\\afsd.dmp");
+    GetLocalTime(&st);
+    StringCbPrintfA(wd, sizeof(wd),
+                    "%s\\afsd-%04d-%02d-%02d-%02d_%02d_%02d.dmp", tmp,
+                    st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
     return CreateFile( wd, GENERIC_WRITE, FILE_SHARE_READ, NULL,
                             CREATE_ALWAYS, FILE_FLAG_WRITE_THROUGH, NULL);
 }
 
-void 
+void
 GenerateMiniDump(PEXCEPTION_POINTERS ep)
 {
     if (IsDebuggerPresent())
         return;
 
-    if (ep == NULL) 
+    if (ep == NULL)
     {
         // Generate exception to get proper context in dump
-        __try 
+        __try
         {
             RaiseException(DBG_CONTINUE, 0, 0, NULL);
-        } 
-        __except(GenerateMiniDump(GetExceptionInformation()), EXCEPTION_CONTINUE_EXECUTION) 
+        }
+        __except(GenerateMiniDump(GetExceptionInformation()), EXCEPTION_CONTINUE_EXECUTION)
         {
         }
-    } 
+    }
     else
     {
         MINIDUMP_EXCEPTION_INFORMATION eInfo;
@@ -1711,27 +1776,27 @@ GenerateMiniDump(PEXCEPTION_POINTERS ep)
 LONG __stdcall afsd_ExceptionFilter(EXCEPTION_POINTERS *ep)
 {
     CONTEXT context;
-#ifdef _DEBUG  
-    BOOL allocRequestBrk = FALSE;
-#endif 
-    HMODULE hLib = NULL;
-  
-    afsi_log("UnhandledException : code : 0x%x, address: 0x%x\n", 
-             ep->ExceptionRecord->ExceptionCode, 
-             ep->ExceptionRecord->ExceptionAddress);
-	   
 #ifdef _DEBUG
-    if (afsd_crtDbgBreakCurrent && 
+    BOOL allocRequestBrk = FALSE;
+#endif
+    HMODULE hLib = NULL;
+
+    afsi_log("UnhandledException : code : 0x%x, address: 0x%x\n",
+             ep->ExceptionRecord->ExceptionCode,
+             ep->ExceptionRecord->ExceptionAddress);
+
+#ifdef _DEBUG
+    if (afsd_crtDbgBreakCurrent &&
         *afsd_crtDbgBreakCurrent == _CrtSetBreakAlloc(*afsd_crtDbgBreakCurrent))
-    { 
+    {
         allocRequestBrk = TRUE;
         afsi_log("Breaking on alloc request # %d\n", *afsd_crtDbgBreakCurrent);
     }
 #endif
-	   
+
     /* save context if we want to print the stack information */
     context = *ep->ContextRecord;
-	   
+
     afsd_printStack(GetCurrentThread(), &context);
 
     GenerateMiniDump(ep);
@@ -1747,15 +1812,15 @@ LONG __stdcall afsd_ExceptionFilter(EXCEPTION_POINTERS *ep)
     if (ep->ExceptionRecord->ExceptionCode == EXCEPTION_BREAKPOINT)
     {
         afsi_log("\nEXCEPTION_BREAKPOINT - continue execution ...\n");
-    
+
 #ifdef _DEBUG
         if (allocRequestBrk)
         {
             afsd_crtDbgBreakCurrent++;
             _CrtSetBreakAlloc(*afsd_crtDbgBreakCurrent);
         }
-#endif         
-#if defined(_X86)    
+#endif
+#if defined(_X86)
         ep->ContextRecord->Eip++;
 #endif
 #if defined(_AMD64_)
@@ -1768,7 +1833,7 @@ LONG __stdcall afsd_ExceptionFilter(EXCEPTION_POINTERS *ep)
         return EXCEPTION_CONTINUE_SEARCH;
     }
 }
-  
+
 void afsd_SetUnhandledExceptionFilter()
 {
 #ifndef NOTRACE
@@ -1782,7 +1847,7 @@ void afsd_DbgBreakAllocInit()
     memset(afsd_crtDbgBreaks, -1, sizeof(afsd_crtDbgBreaks));
     afsd_crtDbgBreakCurrent = afsd_crtDbgBreaks;
 }
-  
+
 void afsd_DbgBreakAdd(DWORD requestNumber)
 {
     int i;
