@@ -1,14 +1,14 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef __CM_DCACHE_ENV__
-#define __CM_DCACHE_ENV__ 1
+#ifndef OPENAFS_WINNT_AFSD_CM_DCACHE_H
+#define OPENAFS_WINNT_AFSD_CM_DCACHE_H 1
 
 /* bulk I/O descriptor */
 typedef struct cm_bulkIO {
@@ -16,7 +16,7 @@ typedef struct cm_bulkIO {
     osi_hyper_t offset;		        /* offset of buffers */
     long length;			/* # of bytes to be transferred */
     int reserved;			/* did we reserve multiple buffers? */
-        
+
     /* all of these buffers are held */
     osi_queueData_t *bufListp;	/* list of buffers involved in I/O */
     osi_queueData_t *bufListEndp;	/* list of buffers involved in I/O */
@@ -31,9 +31,12 @@ extern int cm_HaveBuffer(struct cm_scache *, struct cm_buf *, int haveBufLocked)
 extern long cm_GetBuffer(struct cm_scache *, struct cm_buf *, int *,
 	struct cm_user *, struct cm_req *);
 
-extern long cm_CheckFetchRange(cm_scache_t *scp, osi_hyper_t *startBasep,
-                               osi_hyper_t *length, cm_user_t *up, 
-                               cm_req_t *reqp, osi_hyper_t *realBasep);
+extern long cm_GetData(cm_scache_t *scp, osi_hyper_t *offsetp, char *datap, int data_length,
+                       cm_user_t *userp, cm_req_t *reqp);
+
+extern afs_int32 cm_CheckFetchRange(cm_scache_t *scp, osi_hyper_t *startBasep,
+                                    osi_hyper_t *length, cm_user_t *up,
+                                    cm_req_t *reqp, osi_hyper_t *realBasep);
 
 extern long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
 	cm_bulkIO_t *biop, cm_user_t *up, cm_req_t *reqp);
@@ -50,7 +53,7 @@ extern afs_int32 cm_BkgStore(cm_scache_t *scp, afs_uint32 p1, afs_uint32 p2, afs
 	struct cm_user *userp);
 
 extern void cm_ConsiderPrefetch(cm_scache_t *scp, osi_hyper_t *offsetp,
-                                afs_uint32 count, 
+                                afs_uint32 count,
                                 cm_user_t *userp, cm_req_t *reqp);
 
 extern long cm_ValidateDCache(void);
@@ -60,4 +63,4 @@ extern long cm_ShutdownDCache(void);
 extern long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
                  cm_user_t *userp, cm_req_t *reqp);
 
-#endif /*  __CM_DCACHE_ENV__ */
+#endif /*  OPENAFS_WINNT_AFSD_CM_DCACHE_H */
