@@ -1,7 +1,7 @@
 /*
  * Copyright 2004, Secure Endpoints Inc.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,7 +10,8 @@
 #ifndef CM_MEMMAP_H
 #define CM_MEMMAP_H 1
 
-#define CM_CONFIG_DATA_MAGIC            ('A' | 'F'<<8 | 'S'<<16 | 'D'<<24)
+#define CM_CONFIG_DATA_VERSION  15
+#define CM_CONFIG_DATA_MAGIC            ('A' | 'F'<<8 | 'S'<<16 | CM_CONFIG_DATA_VERSION<<24)
 
 typedef struct cm_config_data {
     afs_uint32          size;
@@ -25,7 +26,7 @@ typedef struct cm_config_data {
     afs_uint32          dirty;
 
     cm_volume_t *       volumeBaseAddress;
-    cm_cell_t   *       cellBaseAddress; 
+    cm_cell_t   *       cellBaseAddress;
     cm_aclent_t *       aclBaseAddress;
     cm_scache_t *       scacheBaseAddress;
     cm_nc_t     *       dnlcBaseAddress;
@@ -38,6 +39,7 @@ typedef struct cm_config_data {
     afs_uint32          maxVolumes;
 
     cm_cell_t	*       allCellsp;
+    cm_cell_t   *       freeCellsp;
     afs_uint32          currentCells;
     afs_uint32          maxCells;
 
@@ -74,7 +76,7 @@ typedef struct cm_config_data {
 
     cm_nc_t 	*       ncfreelist;
     cm_nc_t 	*       nameCache;
-    cm_nc_t 	**      nameHash; 
+    cm_nc_t 	**      nameHash;
 
     cm_buf_t	*       buf_freeListp;
     cm_buf_t    *       buf_freeListEndp;
@@ -115,5 +117,5 @@ PSECURITY_ATTRIBUTES CreateCacheFileSA();
 VOID  FreeCacheFileSA(PSECURITY_ATTRIBUTES psa);
 int   cm_ShutdownMappedMemory(void);
 int   cm_ValidateMappedMemory(char * cachePath);
-int   cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chunkSize, afs_uint64 cacheBlocks, afs_uint32 blockSize);
+int   cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD maxVols, DWORD maxCells, DWORD chunkSize, afs_uint64 cacheBlocks, afs_uint32 blockSize);
 #endif /* CM_MEMMAP_H */
\ No newline at end of file
