@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -14,8 +14,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_kcommon.c,v 1.56.2.18 2009/05/30 17:56:06 shadow Exp $");
 
 #include "rx/rx_kcommon.h"
 
@@ -124,46 +122,6 @@ rxi_GetUDPSocket(u_short port)
     return rxi_GetHostUDPSocket(htonl(INADDR_ANY), port);
 }
 
-#if !defined(AFS_LINUX26_ENV)
-void
-#ifdef AFS_AIX_ENV
-osi_Panic(char *msg, void *a1, void *a2, void *a3)
-#else
-osi_Panic(char *msg, ...)
-#endif
-{
-#ifdef AFS_AIX_ENV
-    if (!msg)
-        msg = "Unknown AFS panic";
-    printf(msg, a1, a2, a3);
-    panic(msg);
-#elif (defined(AFS_DARWIN80_ENV) && !defined(AFS_DARWIN90_ENV)) || (defined(AFS_LINUX22_ENV) && !defined(AFS_LINUX_26_ENV))
-    char buf[256];
-    va_list ap;
-    if (!msg)
-	msg = "Unknown AFS panic";
-
-    va_start(ap, msg);
-    vsnprintf(buf, sizeof(buf), msg, ap);
-    va_end(ap);
-    printf(buf);
-    panic(buf);
-#else
-    va_list ap;
-    if (!msg)
-	msg = "Unknown AFS panic";
-
-    va_start(ap, msg);
-    vprintf(msg, ap);
-    va_end(ap);
-# ifdef AFS_LINUX20_ENV
-    * ((char *) 0) = 0; 
-# else
-    panic(msg);
-# endif
-#endif
-}
-
 /*
  * osi_utoa() - write the NUL-terminated ASCII decimal form of the given
  * unsigned long value into the given buffer.  Returns 0 on success,
@@ -234,6 +192,7 @@ osi_utoa(char *buf, size_t len, unsigned long val)
     return 0;
 }
 
+#ifndef AFS_LINUX26_ENV
 /*
  * osi_AssertFailK() -- used by the osi_Assert() macro.
  *
@@ -284,13 +243,7 @@ osi_AssertFailK(const char *expr, const char *file, int line)
 
 #undef ADDBUF
 
-    osi_Panic(buf);
-}
-#else
-void
-osi_AssertFailK(const char *expr, const char *file, int line)
-{
-    printk(KERN_CRIT "assertion failed: %s, file: %s, line: %d\n", expr, file, line);
+    osi_Panic("%s", buf);
 }
 #endif
 
@@ -302,12 +255,13 @@ rx_ServerProc(void *unused)
 {
     int threadID;
 
-/* jaltman - rxi_dataQuota is protected by a mutex everywhere else */
     rxi_MorePackets(rx_maxReceiveWindow + 2);	/* alloc more packets */
+    MUTEX_ENTER(&rx_quota_mutex);
     rxi_dataQuota += rx_initSendWindow;	/* Reserve some pkts for hard times */
     /* threadID is used for making decisions in GetCall.  Get it by bumping
      * number of threads handling incoming calls */
     threadID = rxi_availProcs++;
+    MUTEX_EXIT(&rx_quota_mutex);
 
 #ifdef RX_ENABLE_LOCKS
     AFS_GUNLOCK();
@@ -395,8 +349,8 @@ MyArrivalProc(struct rx_packet *ahandle,
 void
 rxi_StartListener(void)
 {
+#if !defined(RXK_LISTENER_ENV) && !defined(RXK_UPCALL_ENV)
     /* if kernel, give name of appropriate procedures */
-#ifndef RXK_LISTENER_ENV
     rxk_GetPacketProc = MyPacketProc;
     rxk_PacketArrivalProc = MyArrivalProc;
     rxk_init();
@@ -409,22 +363,20 @@ void
 rxi_InitPeerParams(struct rx_peer *pp)
 {
     u_short rxmtu;
-    afs_int32 i, mtu;
 
 #ifdef	ADAPT_MTU
-#ifndef AFS_SUN5_ENV
-#ifdef AFS_USERSPACE_IP_ADDR
+# ifndef AFS_SUN5_ENV
+#  ifdef AFS_USERSPACE_IP_ADDR
+    afs_int32 i;
+    afs_int32 mtu;
+
     i = rxi_Findcbi(pp->host);
     if (i == -1) {
-	pp->timeout.sec = 3;
-	/* pp->timeout.usec = 0; */
+	rx_rto_setPeerTimeoutSecs(pp, 3);
 	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
     } else {
-	pp->timeout.sec = 2;
-	/* pp->timeout.usec = 0; */
+	rx_rto_setPeerTimeoutSecs(pp, 2);
 	pp->ifMTU = MIN(RX_MAX_PACKET_SIZE, rx_MyMaxSendSize);
-    }
-    if (i != -1) {
 	mtu = ntohl(afs_cb_interface.mtu[i]);
 	/* Diminish the packet size to one based on the MTU given by
 	 * the interface. */
@@ -433,56 +385,50 @@ rxi_InitPeerParams(struct rx_peer *pp)
 	    if (rxmtu < pp->ifMTU)
 		pp->ifMTU = rxmtu;
 	}
-    } else {			/* couldn't find the interface, so assume the worst */
-	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
     }
-#else /* AFS_USERSPACE_IP_ADDR */
-    AFS_IFNET_T ifn;
+#  else /* AFS_USERSPACE_IP_ADDR */
+    rx_ifnet_t ifn;
 
-#if !defined(AFS_SGI62_ENV)
+#   if !defined(AFS_SGI62_ENV)
     if (numMyNetAddrs == 0)
 	(void)rxi_GetIFInfo();
-#endif
+#   endif
 
     ifn = rxi_FindIfnet(pp->host, NULL);
     if (ifn) {
-	pp->timeout.sec = 2;
-	/* pp->timeout.usec = 0; */
+	rx_rto_setPeerTimeoutSecs(pp, 2);
 	pp->ifMTU = MIN(RX_MAX_PACKET_SIZE, rx_MyMaxSendSize);
-#ifdef IFF_POINTOPOINT
-	if (ifnet_flags(ifn) & IFF_POINTOPOINT) {
+#   ifdef IFF_POINTOPOINT
+	if (rx_ifnet_flags(ifn) & IFF_POINTOPOINT) {
 	    /* wish we knew the bit rate and the chunk size, sigh. */
-	    pp->timeout.sec = 4;
+	    rx_rto_setPeerTimeoutSecs(pp, 4);
 	    pp->ifMTU = RX_PP_PACKET_SIZE;
 	}
-#endif /* IFF_POINTOPOINT */
+#   endif /* IFF_POINTOPOINT */
 	/* Diminish the packet size to one based on the MTU given by
 	 * the interface. */
-	if (ifnet_mtu(ifn) > (RX_IPUDP_SIZE + RX_HEADER_SIZE)) {
-	    rxmtu = ifnet_mtu(ifn) - RX_IPUDP_SIZE;
+	if (rx_ifnet_mtu(ifn) > (RX_IPUDP_SIZE + RX_HEADER_SIZE)) {
+	    rxmtu = rx_ifnet_mtu(ifn) - RX_IPUDP_SIZE;
 	    if (rxmtu < pp->ifMTU)
 		pp->ifMTU = rxmtu;
 	}
     } else {			/* couldn't find the interface, so assume the worst */
-	pp->timeout.sec = 3;
-	/* pp->timeout.usec = 0; */
+	rx_rto_setPeerTimeoutSecs(pp, 3);
 	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
     }
-#endif /* else AFS_USERSPACE_IP_ADDR */
-#else /* AFS_SUN5_ENV */
+#  endif /* else AFS_USERSPACE_IP_ADDR */
+# else /* AFS_SUN5_ENV */
+    afs_int32 mtu;
+
     mtu = rxi_FindIfMTU(pp->host);
 
     if (mtu <= 0) {
-	pp->timeout.sec = 3;
-	/* pp->timeout.usec = 0; */
+	rx_rto_setPeerTimeoutSecs(pp, 3);
 	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
     } else {
-	pp->timeout.sec = 2;
-	/* pp->timeout.usec = 0; */
+	rx_rto_setPeerTimeoutSecs(pp, 2);
 	pp->ifMTU = MIN(RX_MAX_PACKET_SIZE, rx_MyMaxSendSize);
-    }
 
-    if (mtu > 0) {
 	/* Diminish the packet size to one based on the MTU given by
 	 * the interface. */
 	if (mtu > (RX_IPUDP_SIZE + RX_HEADER_SIZE)) {
@@ -490,13 +436,11 @@ rxi_InitPeerParams(struct rx_peer *pp)
 	    if (rxmtu < pp->ifMTU)
 		pp->ifMTU = rxmtu;
 	}
-    } else {			/* couldn't find the interface, so assume the worst */
-	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE,rx_MyMaxSendSize);
     }
-#endif /* AFS_SUN5_ENV */
+# endif /* AFS_SUN5_ENV */
 #else /* ADAPT_MTU */
     pp->rateFlag = 2;		/* start timing after two full packets */
-    pp->timeout.sec = 2;
+    rx_rto_setPeerTimeoutSecs(pp, 2);
     pp->ifMTU = OLD_MAX_PACKET_SIZE;
 #endif /* else ADAPT_MTU */
     pp->ifMTU = rxi_AdjustIfMTU(pp->ifMTU);
@@ -576,7 +520,7 @@ rxi_GetcbiInfo(void)
 	    rxmtu * rxi_nRecvFrags + ((rxi_nRecvFrags - 1) * UDP_HDR_SIZE);
 	maxmtu = rxi_AdjustMaxMTU(rxmtu, maxmtu);
 	addrs[i++] = ifinaddr;
-	if ((ifinaddr != 0x7f000001) && (maxmtu > rx_maxReceiveSize)) {
+	if (!rx_IsLoopbackAddr(ifinaddr) && (maxmtu > rx_maxReceiveSize)) {
 	    rx_maxReceiveSize = MIN(RX_MAX_PACKET_SIZE, maxmtu);
 	    rx_maxReceiveSize = MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
 	}
@@ -670,27 +614,27 @@ rxi_GetIFInfo(void)
     errno_t t;
     unsigned int count;
     int cnt=0, m, j;
-    ifaddr_t *ifads;
-    ifnet_t *ifn;
+    rx_ifaddr_t *ifads;
+    rx_ifnet_t *ifns;
     struct sockaddr sout;
     struct sockaddr_in *sin;
     struct in_addr pin;
 #else
-    struct ifaddr *ifad;	/* ifnet points to a if_addrlist of ifaddrs */
-    struct ifnet *ifn;
+    rx_ifaddr_t ifad;	/* ifnet points to a if_addrlist of ifaddrs */
+    rx_ifnet_t ifn;
 #endif
 
     memset(addrs, 0, sizeof(addrs));
     memset(mtus, 0, sizeof(mtus));
 
 #if defined(AFS_DARWIN80_ENV)
-    if (!ifnet_list_get(AF_INET, &ifn, &count)) {
+    if (!ifnet_list_get(AF_INET, &ifns, &count)) {
 	for (m = 0; m < count; m++) {
-	    if (!ifnet_get_address_list(ifn[m], &ifads)) {
+	    if (!ifnet_get_address_list(ifns[m], &ifads)) {
 		for (j = 0; ifads[j] != NULL && cnt < ADDRSPERSITE; j++) {
 		    if ((t = ifaddr_address(ifads[j], &sout, sizeof(struct sockaddr))) == 0) {
 			sin = (struct sockaddr_in *)&sout;
-			rxmtu = ifnet_mtu(ifaddr_ifnet(ifads[j])) - RX_IPUDP_SIZE;
+			rxmtu = rx_ifnet_mtu(rx_ifaddr_ifnet(ifads[j])) - RX_IPUDP_SIZE;
 			ifinaddr = ntohl(sin->sin_addr.s_addr);
 			if (myNetAddrs[i] != ifinaddr) {
 			    different++;
@@ -702,9 +646,9 @@ rxi_GetIFInfo(void)
 			    ((rxi_nRecvFrags - 1) * UDP_HDR_SIZE);
 			maxmtu = rxi_AdjustMaxMTU(rxmtu, maxmtu);
 			addrs[i++] = ifinaddr;
-			if ((ifinaddr != 0x7f000001) && 
+			if (!rx_IsLoopbackAddr(ifinaddr) &&
 			    (maxmtu > rx_maxReceiveSize)) {
-			    rx_maxReceiveSize = 
+			    rx_maxReceiveSize =
 				MIN(RX_MAX_PACKET_SIZE, maxmtu);
 			    rx_maxReceiveSize =
 				MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
@@ -715,14 +659,18 @@ rxi_GetIFInfo(void)
 		ifnet_free_address_list(ifads);
 	    }
 	}
-	ifnet_list_free(ifn);
+	ifnet_list_free(ifns);
     }
 #else
 #if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+#if defined(AFS_FBSD80_ENV)
+    TAILQ_FOREACH(ifn, &V_ifnet, if_link) {
+#else
     TAILQ_FOREACH(ifn, &ifnet, if_link) {
+#endif
 	if (i >= ADDRSPERSITE)
 	    break;
-#elif defined(AFS_OBSD_ENV)
+#elif defined(AFS_OBSD_ENV) || defined(AFS_NBSD_ENV)
     for (ifn = ifnet.tqh_first; i < ADDRSPERSITE && ifn != NULL;
 	 ifn = ifn->if_list.tqe_next) {
 #else
@@ -733,7 +681,7 @@ rxi_GetIFInfo(void)
 	TAILQ_FOREACH(ifad, &ifn->if_addrhead, ifa_link) {
 	    if (i >= ADDRSPERSITE)
 		break;
-#elif defined(AFS_OBSD_ENV)
+#elif defined(AFS_OBSD_ENV) || defined(AFS_NBSD_ENV)
 	for (ifad = ifn->if_addrlist.tqh_first;
 	     ifad != NULL && i < ADDRSPERSITE;
 	     ifad = ifad->ifa_list.tqe_next) {
@@ -755,7 +703,7 @@ rxi_GetIFInfo(void)
 		    ((rxi_nRecvFrags - 1) * UDP_HDR_SIZE);
 		maxmtu = rxi_AdjustMaxMTU(rxmtu, maxmtu);
 		addrs[i++] = ifinaddr;
-		if ((ifinaddr != 0x7f000001) && (maxmtu > rx_maxReceiveSize)) {
+		if (!rx_IsLoopbackAddr(ifinaddr) && (maxmtu > rx_maxReceiveSize)) {
 		    rx_maxReceiveSize = MIN(RX_MAX_PACKET_SIZE, maxmtu);
 		    rx_maxReceiveSize =
 			MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
@@ -780,47 +728,29 @@ rxi_GetIFInfo(void)
     return different;
 }
 
-#if defined(AFS_DARWIN60_ENV) || defined(AFS_XBSD_ENV)
+#if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 /* Returns ifnet which best matches address */
-#ifdef AFS_DARWIN80_ENV
-ifnet_t
-#else
-struct ifnet *
-#endif
+rx_ifnet_t
 rxi_FindIfnet(afs_uint32 addr, afs_uint32 * maskp)
 {
     struct sockaddr_in s, sr;
-#ifdef AFS_DARWIN80_ENV
-    ifaddr_t ifad;
-#else
-    struct ifaddr *ifad;
-#endif
+    rx_ifaddr_t ifad;
 
     s.sin_family = AF_INET;
     s.sin_addr.s_addr = addr;
-#ifdef AFS_DARWIN80_ENV
-    ifad = ifaddr_withnet((struct sockaddr *)&s);
-#else
-    ifad = ifa_ifwithnet((struct sockaddr *)&s);
-#endif
+    ifad = rx_ifaddr_withnet((struct sockaddr *)&s);
 
-#ifdef AFS_DARWIN80_ENV
     if (ifad && maskp) {
-	ifaddr_netmask(ifad, (struct sockaddr *)&sr, sizeof(sr));
+	rx_ifaddr_netmask(ifad, (struct sockaddr *)&sr, sizeof(sr));
 	*maskp = sr.sin_addr.s_addr;
     }
-    return (ifad ? ifaddr_ifnet(ifad) : NULL);
-#else
-    if (ifad && maskp)
-	*maskp = ((struct sockaddr_in *)ifad->ifa_netmask)->sin_addr.s_addr;
-    return (ifad ? ifad->ifa_ifp : NULL);
-#endif
+    return (ifad ? rx_ifaddr_ifnet(ifad) : NULL);
 }
 
-#else /* DARWIN60 || XBSD */
+#else /* DARWIN || XBSD */
 
 /* Returns ifnet which best matches address */
-struct ifnet *
+rx_ifnet_t
 rxi_FindIfnet(afs_uint32 addr, afs_uint32 * maskp)
 {
     int match_value = 0;
@@ -860,7 +790,7 @@ rxi_FindIfnet(afs_uint32 addr, afs_uint32 * maskp)
 	*maskp = ifad->ia_subnetmask;
     return (ifad ? ifad->ia_ifp : NULL);
 }
-#endif /* else DARWIN60 || XBSD */
+#endif /* else DARWIN || XBSD */
 #endif /* else AFS_USERSPACE_IP_ADDR */
 #endif /* !SUN5 && !SGI62 */
 
@@ -884,7 +814,7 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
 #else
     struct socket *newSocket;
 #endif
-#if (!defined(AFS_HPUX1122_ENV) && !defined(AFS_FBSD50_ENV))
+#if (!defined(AFS_HPUX1122_ENV) && !defined(AFS_FBSD_ENV))
     struct mbuf *nam;
 #endif
     struct sockaddr_in myaddr;
@@ -908,7 +838,7 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     AFS_GUNLOCK();
 #if	defined(AFS_HPUX102_ENV)
 #if     defined(AFS_HPUX110_ENV)
-    /* we need a file associated with the socket so sosend in NetSend 
+    /* we need a file associated with the socket so sosend in NetSend
      * will not fail */
     /* blocking socket */
     code = socreate(AF_INET, &newSocket, SOCK_DGRAM, 0, 0);
@@ -927,13 +857,19 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
 #endif /* else AFS_HPUX110_ENV */
 #elif defined(AFS_SGI65_ENV) || defined(AFS_OBSD_ENV)
     code = socreate(AF_INET, &newSocket, SOCK_DGRAM, IPPROTO_UDP);
-#elif defined(AFS_FBSD50_ENV)
+#elif defined(AFS_FBSD_ENV)
     code = socreate(AF_INET, &newSocket, SOCK_DGRAM, IPPROTO_UDP,
 		    afs_osi_credp, curthread);
-#elif defined(AFS_FBSD40_ENV)
-    code = socreate(AF_INET, &newSocket, SOCK_DGRAM, IPPROTO_UDP, curproc);
 #elif defined(AFS_DARWIN80_ENV)
+#ifdef RXK_LISTENER_ENV
     code = sock_socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP, NULL, NULL, &newSocket);
+#else
+    code = sock_socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP, rx_upcall, NULL, &newSocket);
+#endif
+#elif defined(AFS_NBSD50_ENV)
+    code = socreate(AF_INET, &newSocket, SOCK_DGRAM, 0, osi_curproc(), NULL);
+#elif defined(AFS_NBSD40_ENV)
+    code = socreate(AF_INET, &newSocket, SOCK_DGRAM, 0, osi_curproc());
 #else
     code = socreate(AF_INET, &newSocket, SOCK_DGRAM, 0);
 #endif /* AFS_HPUX102_ENV */
@@ -957,8 +893,11 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     memcpy((caddr_t) bindnam->b_rptr + SO_MSGOFFSET, (caddr_t) & myaddr,
 	   addrsize);
     bindnam->b_wptr = bindnam->b_rptr + (addrsize + SO_MSGOFFSET + 1);
-
+#if defined(AFS_NBSD40_ENV)
+    code = sobind(newSocket, bindnam, addrsize, osi_curproc());
+#else
     code = sobind(newSocket, bindnam, addrsize);
+#endif
     if (code) {
 	soclose(newSocket);
 #if !defined(AFS_HPUX1122_ENV)
@@ -970,7 +909,7 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     freeb(bindnam);
 #else /* AFS_HPUX110_ENV */
 #if defined(AFS_DARWIN80_ENV)
-    { 
+    {
        int buflen = 50000;
        int i,code2;
        for (i=0;i<2;i++) {
@@ -994,17 +933,14 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     }
 #endif
 #if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
-#if defined(AFS_FBSD50_ENV)
+#if defined(AFS_FBSD_ENV)
     code = sobind(newSocket, (struct sockaddr *)&myaddr, curthread);
-#elif defined(AFS_FBSD40_ENV)
-    code = sobind(newSocket, (struct sockaddr *)&myaddr, curproc);
 #else
     code = sobind(newSocket, (struct sockaddr *)&myaddr);
 #endif
     if (code) {
 	dpf(("sobind fails (%d)\n", (int)code));
 	soclose(newSocket);
-	AFS_GLOCK();
 	goto bad;
     }
 #else /* defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV) */
@@ -1025,7 +961,7 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     BHV_PDATA(&bhv) = (void *)newSocket;
     code = sobind(&bhv, nam);
     m_freem(nam);
-#elif defined(AFS_OBSD44_ENV)
+#elif defined(AFS_OBSD44_ENV) || defined(AFS_NBSD40_ENV)
     code = sobind(newSocket, nam, osi_curproc());
 #else
     code = sobind(newSocket, nam);
@@ -1073,8 +1009,8 @@ rxk_FreeSocket(struct socket *asocket)
     if (asocket->so_fp) {
 	struct file *fp = asocket->so_fp;
 #if !defined(AFS_HPUX1123_ENV)
-	/* 11.23 still has falloc, but not FPENTRYFREE ! 
-	 * so for now if we shutdown, we will waist a file 
+	/* 11.23 still has falloc, but not FPENTRYFREE !
+	 * so for now if we shutdown, we will waist a file
 	 * structure */
 	FPENTRYFREE(fp);
 	asocket->so_fp = NULL;
@@ -1089,7 +1025,21 @@ rxk_FreeSocket(struct socket *asocket)
 }
 #endif /* !SUN5 && !LINUX20 */
 
-#if defined(RXK_LISTENER_ENV) || defined(AFS_SUN5_ENV)
+#if defined(RXK_LISTENER_ENV) || defined(AFS_SUN5_ENV) || defined(RXK_UPCALL_ENV)
+#ifdef RXK_TIMEDSLEEP_ENV
+/* Shutting down should wake us up, as should an earlier event. */
+void
+rxi_ReScheduleEvents(void)
+{
+    /* needed to allow startup */
+    int glock = ISAFS_GLOCK();
+    if (!glock)
+	AFS_GLOCK();
+    osi_rxWakeup(&afs_termState);
+    if (!glock)
+        AFS_GUNLOCK();
+}
+#endif
 /*
  * Run RX event daemon every second (5 times faster than rest of systems)
  */
@@ -1113,7 +1063,12 @@ afs_rxevent_daemon(void)
 	afs_Trace1(afs_iclSetp, CM_TRACE_TIMESTAMP, ICL_TYPE_STRING,
 		   "before afs_osi_Wait()");
 #endif
+#ifdef RXK_TIMEDSLEEP_ENV
+	afs_osi_TimedSleep(&afs_termState, MAX(500, ((temp.sec * 1000) +
+						     (temp.usec / 1000))), 0);
+#else
 	afs_osi_Wait(500, NULL, 0);
+#endif
 #ifdef RX_KERNEL_TRACE
 	afs_Trace1(afs_iclSetp, CM_TRACE_TIMESTAMP, ICL_TYPE_STRING,
 		   "after afs_osi_Wait()");
@@ -1121,12 +1076,10 @@ afs_rxevent_daemon(void)
 	if (afs_termState == AFSOP_STOP_RXEVENT) {
 #ifdef RXK_LISTENER_ENV
 	    afs_termState = AFSOP_STOP_RXK_LISTENER;
-#else
-#ifdef AFS_SUN510_ENV
+#elif defined(AFS_SUN510_ENV) || defined(RXK_UPCALL_ENV)
 	    afs_termState = AFSOP_STOP_NETIF;
 #else
 	    afs_termState = AFSOP_STOP_COMPLETE;
-#endif
 #endif
 	    osi_rxWakeup(&afs_termState);
 	    return;
@@ -1163,7 +1116,7 @@ rxk_ReadPacket(osi_socket so, struct rx_packet *p, int *host, int *port)
     } else
 	tlen = rlen;
 
-    /* add some padding to the last iovec, it's just to make sure that the 
+    /* add some padding to the last iovec, it's just to make sure that the
      * read doesn't return more data than we expect, and is done to get around
      * our problems caused by the lack of a length field in the rx header. */
     savelen = p->wirevec[p->niovecs - 1].iov_len;
@@ -1229,7 +1182,7 @@ rxk_ReadPacket(osi_socket so, struct rx_packet *p, int *host, int *port)
 	return code;
 }
 
-/* rxk_Listener() 
+/* rxk_Listener()
  *
  * Listen for packets on socket. This thread is typically started after
  * rx_Init has called rxi_StartListener(), but nevertheless, ensures that
@@ -1293,6 +1246,9 @@ rxk_Listener(void)
     AFS_GUNLOCK();
 #endif /* RX_ENABLE_LOCKS && !AFS_SUN5_ENV */
     while (afs_termState != AFSOP_STOP_RXK_LISTENER) {
+        /* See if a check for additional packets was issued */
+        rx_CheckPackets();
+
 	if (rxp) {
 	    rxi_RestoreDataBufs(rxp);
 	} else {
@@ -1321,7 +1277,7 @@ rxk_Listener(void)
     rxk_ListenerTask = 0;
     osi_rxWakeup(&rxk_ListenerTask);
 #endif
-#if defined(AFS_SUN5_ENV)
+#if defined(AFS_SUN5_ENV) || defined(AFS_FBSD_ENV)
     osi_rxWakeup(&rxk_ListenerPid);
 #endif
 #ifdef AFS_SUN5_ENV
@@ -1340,5 +1296,61 @@ osi_StopListener(void)
 }
 #endif
 #endif /* RXK_LISTENER_ENV */
-
 #endif /* !NCR && !UKERNEL */
+
+#if !defined(AFS_LINUX26_ENV)
+void
+#if defined(AFS_AIX_ENV)
+osi_Panic(char *msg, void *a1, void *a2, void *a3)
+#else
+osi_Panic(char *msg, ...)
+#endif
+{
+#ifdef AFS_AIX_ENV
+    if (!msg)
+	msg = "Unknown AFS panic";
+    /*
+     * we should probably use the errsave facility here. it is not
+     * varargs-aware
+     */
+
+    printf(msg, a1, a2, a3);
+    panic(msg);
+#elif defined(AFS_SGI_ENV)
+    va_list ap;
+
+    /* Solaris has vcmn_err, Sol10 01/06 may have issues. Beware. */
+    if (!msg) {
+	cmn_err(CE_PANIC, "Unknown AFS panic");
+    } else {
+	va_start(ap, msg);
+	icmn_err(CE_PANIC, msg, ap);
+	va_end(ap);
+    }
+#elif defined(AFS_DARWIN80_ENV) || (defined(AFS_LINUX22_ENV) && !defined(AFS_LINUX_26_ENV))
+    char buf[256];
+    va_list ap;
+    if (!msg)
+	msg = "Unknown AFS panic";
+
+    va_start(ap, msg);
+    vsnprintf(buf, sizeof(buf), msg, ap);
+    va_end(ap);
+    printf("%s", buf);
+    panic(buf);
+#else
+    va_list ap;
+    if (!msg)
+	msg = "Unknown AFS panic";
+
+    va_start(ap, msg);
+    vprintf(msg, ap);
+    va_end(ap);
+# ifdef AFS_LINUX20_ENV
+    * ((char *) 0) = 0;
+# else
+    panic(msg);
+# endif
+#endif
+}
+#endif
